@@ -148,7 +148,7 @@ int prelude_read_multiline(FILE *fd, unsigned int *line, char *buf, size_t size)
         size_t i;
 
         if ( ! fgets(buf, size, fd) )
-                return -1;
+                return prelude_error(PRELUDE_ERROR_EOF);
 
         (*line)++;
          
@@ -172,6 +172,42 @@ int prelude_read_multiline(FILE *fd, unsigned int *line, char *buf, size_t size)
 
 
 
+/**
+ * prelude_read_multiline2:
+ * @fd: File descriptor to read input from.
+ * @line: Pointer to a line counter.
+ * @out: Pointer to a #prelude_string_t object where the line should be stored.
+ *
+ * This function handles line reading separated by the '\' character.
+ *
+ * Returns: 0 on success, a negative value if an error occured.
+ */
+int prelude_read_multiline2(FILE *fd, unsigned int *line, prelude_string_t *out) 
+{
+        int ret;
+        char buf[8192];
+        
+        prelude_string_clear(out);
+
+        do {
+                ret = prelude_read_multiline(fd, line, buf, sizeof(buf));
+                if ( ret < 0 )
+                        return ret;
+
+                ret = prelude_string_cat(out, buf);
+                if ( ret < 0 )
+                        break;
+                
+                if ( buf[strlen(buf) - 1 ] == '\n' )
+                        break;
+                
+        } while ( 1 );
+
+        return 0;
+}
+
+
+
 
 /**
  * prelude_hton64:
@@ -343,13 +379,21 @@ time_t prelude_timegm(struct tm *tm)
 void *prelude_sockaddr_get_inaddr(struct sockaddr *sa) 
 {
         void *ret = NULL;
-        
+        union {
+                struct sockaddr *sa;
+                struct sockaddr_in *sa4;
+#ifdef HAVE_IPV6
+                struct sockaddr_in6 *sa6;
+#endif
+        } val;
+
+        val.sa = sa;
         if ( sa->sa_family == AF_INET )
-                ret = &((struct sockaddr_in *) sa)->sin_addr;
+                ret = &val.sa4->sin_addr;
 
 #ifdef HAVE_IPV6
         else if ( sa->sa_family == AF_INET6 )
-                ret = &((struct sockaddr_in6 *) sa)->sin6_addr;
+                ret = &val.sa6->sin6_addr;
 #endif
         
         return ret;
@@ -420,21 +464,41 @@ prelude_msg_priority_t _idmef_impact_severity_to_msg_priority(idmef_impact_sever
 
 
 
-int _idmef_message_assign_messageid(idmef_message_t *msg, prelude_ident_t *ident)
+int _idmef_message_assign_missing(prelude_client_t *client, idmef_message_t *msg)
 {
+        int ret;
+        idmef_time_t *time;
         idmef_alert_t *alert;
         idmef_heartbeat_t *heartbeat;
+        prelude_ident_t *ident = prelude_client_get_unique_ident(client);
         
         if ( idmef_message_get_type(msg) == IDMEF_MESSAGE_TYPE_ALERT ) {
                 alert = idmef_message_get_alert(msg);
                 
                 if ( ! idmef_alert_get_messageid(alert) )
                         idmef_alert_set_messageid(alert, get_message_ident(ident));
+
+                if ( ! idmef_alert_get_create_time(alert) ) {
+                        ret = idmef_time_new_from_gettimeofday(&time);
+                        if ( ret < 0 )
+                                return ret;
+
+                        idmef_alert_set_create_time(alert, time);
+                }
+                
         } else {       
                 heartbeat = idmef_message_get_heartbeat(msg);
 
                 if ( ! idmef_heartbeat_get_messageid(heartbeat) )
                         idmef_heartbeat_set_messageid(heartbeat, get_message_ident(ident));
+
+                if ( ! idmef_heartbeat_get_create_time(heartbeat) ) {
+                        ret = idmef_time_new_from_gettimeofday(&time);
+                        if ( ret < 0 )
+                                return ret;
+
+                        idmef_heartbeat_set_create_time(heartbeat, time);
+                }
         }
 
         return 0;
