@@ -29,6 +29,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <stdarg.h>
+#include <limits.h>
 
 #include "prelude-thread.h"
 #include "prelude-hash.h"
@@ -54,9 +55,8 @@
 #define MAX_DEPTH     16
 #define MAX_NAME_LEN 128
 
-#define INDEX_UNDEFINED -2
-#define INDEX_FORBIDDEN -3
-
+#define INDEX_UNDEFINED INT_MIN
+#define INDEX_FORBIDDEN (INT_MIN + 1)
 
 
 typedef struct idmef_path_element {
@@ -231,18 +231,23 @@ static int idmef_path_get_nth_internal(idmef_value_t **value, idmef_path_t *path
                                        idmef_class_id_t parent_class, int which)
 {
 	unsigned int cnt = 0;
-	prelude_list_t *tmp, *last = NULL;
-        
-	prelude_list_for_each(list, tmp) {                
-		if ( cnt++ == which )
-			return idmef_path_get_internal(value, path, depth, tmp, parent_class);
+	prelude_list_t *tmp;
 
-                last = tmp;
+        if ( which >= 0 ) {
+                prelude_list_for_each(list, tmp) {                
+                        if ( cnt++ == which )
+                                return idmef_path_get_internal(value, path, depth, tmp, parent_class);
+                }
+        } else {                
+                which = -which;
+                which--; /* With negative value, -1 is the base, translate to 0 */
+                
+                prelude_list_for_each_reversed(list, tmp) {                
+                        if ( cnt++ == which )
+                                return idmef_path_get_internal(value, path, depth, tmp, parent_class);
+                }
         }
-
-        if ( which == -1 && last )
-                return idmef_path_get_internal(value, path, depth, last, parent_class);
-
+        
 	return 0;
 }
 
@@ -332,8 +337,10 @@ int idmef_path_set(idmef_path_t *path, idmef_message_t *message, idmef_value_t *
                 elem = &path->elem[i];
                 
 	    	if ( elem->index == INDEX_UNDEFINED && idmef_class_is_child_list(class, elem->position) )
-			return prelude_error(PRELUDE_ERROR_IDMEF_PATH_MISS_INDEX);
-
+                        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_MISS_INDEX,
+                                                     "IDMEF path element '%s' need indexing",
+                                                     idmef_class_get_name(elem->class));
+                
                 ret = idmef_class_new_child(ptr, class, elem->position, elem->index, &child);
                 if ( ret < 0 )
 		    	return ret;
@@ -445,7 +452,14 @@ static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
                 ptr2 = strchr(ptr, '(');
                 if ( ptr2 ) {
                         *ptr2 = '\0';
-                        index = strtol(ptr2 + 1, NULL, 0);
+                        if ( strncmp(ptr2 + 1, "<<", 2) == 0 )
+                                index = IDMEF_LIST_PREPEND;
+                        
+                        else if ( strncmp(ptr2 + 1, ">>", 2) == 0 )
+                                index = IDMEF_LIST_APPEND;
+
+                        else if ( strncmp(ptr2 + 1, "*", 1) != 0 )
+                                index = strtol(ptr2 + 1, NULL, 0);
                 }
                 
                 child = idmef_class_find_child(class, ptr);
@@ -460,8 +474,9 @@ static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
                         *ptr2 = '(';
                         
                         if ( ! idmef_class_is_child_list(class, child) )
-			    	return prelude_error(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN);
-
+			    	return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN,
+                                                             "Invalid IDMEF path element '%s': indexing not supported", ptr);
+                        
                         path->elem[depth].index = index;
                 }
 
@@ -761,7 +776,7 @@ int idmef_path_undefine_index(idmef_path_t *path, unsigned int depth)
  * Returns: The element index, or a negative value if an error occured.
  */
 int idmef_path_get_index(const idmef_path_t *path, unsigned int depth)
-{
+{        
         if ( depth > (path->depth - 1) )
                 return prelude_error(PRELUDE_ERROR_IDMEF_PATH_DEPTH);
         
@@ -770,7 +785,7 @@ int idmef_path_get_index(const idmef_path_t *path, unsigned int depth)
         
         if ( path->elem[depth].index == INDEX_FORBIDDEN )
                 return prelude_error(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN);
-
+        
         return path->elem[depth].index;
 }
 
