@@ -40,7 +40,7 @@
 #include "idmef-value-type.h"
 
 
-#define OBJECT_OPERATOR  IDMEF_CRITERION_OPERATOR_NULL|IDMEF_CRITERION_OPERATOR_NOT
+#define CLASS_OPERATOR  IDMEF_CRITERION_OPERATOR_NULL|IDMEF_CRITERION_OPERATOR_NOT
 
 #define DATA_OPERATOR    IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOT| \
                          IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_GREATER
@@ -58,7 +58,10 @@
 #define GENERIC_ONE_BASE_RW_FUNC(scanfmt, printfmt, name, type)                          \
         static int name ## _read(idmef_value_type_t *dst, const char *buf)               \
         {                                                                                \
-                return sscanf(buf, (scanfmt), &(dst)->data. name ##_val) == 1 ? 0 : -1;  \
+                int ret;                                                                 \
+                ret = sscanf(buf, (scanfmt), &(dst)->data. name ##_val);                 \
+                return (ret == 1) ? 0 : prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, \
+                                                              "Reading " #name " value failed");    \
         }                                                                                \
                                                                                          \
         static int name ## _write(const idmef_value_type_t *src, prelude_string_t *out)  \
@@ -77,7 +80,8 @@
 		else                                                                    \
 		        ret = sscanf(buf, (fmt_dec), &(dst)->data. name ##_val);        \
                                                                                         \
-                return (ret <= 0) ? -1 : 0;                                             \
+                return (ret == 1) ? 0 : prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, \
+                                                              "Reading " #name " value failed");    \
 	}										\
 											\
         static int name ## _write(const idmef_value_type_t *src, prelude_string_t *out)	\
@@ -88,8 +92,10 @@
 
 
 typedef struct {
+        const char *name;
+        
         size_t len;
-
+        
         idmef_criterion_operator_t operator;
         
         int (*copy)(const idmef_value_type_t *src, void *dst, size_t size);
@@ -110,9 +116,11 @@ static int byte_read(idmef_value_type_t *dst, const char *buf, unsigned int min,
         char *endptr;
         long int tmp;
 
-        tmp = strtol(buf, &endptr, 0);
+        tmp = strtol(buf, &endptr, 0);        
         if ( tmp < min || tmp > max )
-                return -1;
+                return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE,
+                                             "Value out of range, required: [%u-%u], got %s",
+                                             min, max, buf);
         
         dst->data.int8_val = (int8_t) tmp;
 
@@ -232,8 +240,9 @@ static int time_read(idmef_value_type_t *dst, const char *buf)
 	ret = idmef_time_new_from_string(&dst->data.time_val, buf);
         if ( ret == 0 )
                 return 0;
-        
-        return prelude_error(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE);
+
+        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE,
+                                     "Invalid time format specified: '%s'", buf);
 }
 
 
@@ -397,37 +406,37 @@ static void data_destroy(idmef_value_type_t *type)
 
 
 static const idmef_value_type_operation_t ops_tbl[] = {
-        { 0, 0, NULL, NULL, NULL, NULL, NULL, NULL},
-        { sizeof(int8_t), INTEGER_OPERATOR, generic_copy,
+        { "unknown", 0, 0, NULL, NULL, NULL, NULL, NULL, NULL                     },
+        { "int8", sizeof(int8_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, int8_read, int8_write             },
-        { sizeof(uint8_t), INTEGER_OPERATOR, generic_copy,
+        { "uint8", sizeof(uint8_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, uint8_read, uint8_write           },
-        { sizeof(int16_t), INTEGER_OPERATOR, generic_copy,
+        { "int16", sizeof(int16_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, int16_read, int16_write           },
-        { sizeof(uint16_t), INTEGER_OPERATOR, generic_copy,
+        { "uint16", sizeof(uint16_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, uint16_read, uint16_write         },
-        { sizeof(int32_t), INTEGER_OPERATOR, generic_copy,
+        { "int32", sizeof(int32_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, int32_read, int32_write           },
-        { sizeof(uint32_t), INTEGER_OPERATOR, generic_copy,
+        { "uint32", sizeof(uint32_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, uint32_read, uint32_write         },
-        { sizeof(int64_t), INTEGER_OPERATOR, generic_copy,
+        { "int64", sizeof(int64_t), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, int64_read, int64_write           },
-        { sizeof(uint64_t), INTEGER_OPERATOR, generic_copy, 
+        { "uint64", sizeof(uint64_t), INTEGER_OPERATOR, generic_copy, 
           generic_clone, NULL, generic_compare, uint64_read, uint64_write         },
-        { sizeof(float), INTEGER_OPERATOR, generic_copy,
+        { "float", sizeof(float), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, float_read, float_write           },
-        { sizeof(double), INTEGER_OPERATOR, generic_copy,
+        { "double", sizeof(double), INTEGER_OPERATOR, generic_copy,
           generic_clone, NULL, generic_compare, double_read, double_write         },
-        { 0, STRING_OPERATOR, string_copy,
+        { "string", 0, STRING_OPERATOR, string_copy,
           string_clone, string_destroy, string_compare, string_read, string_write },
-        { 0, TIME_OPERATOR, time_copy,
+        { "time", 0, TIME_OPERATOR, time_copy,
           time_clone, time_destroy, time_compare, time_read, time_write           }, 
-        { 0, DATA_OPERATOR, data_copy,
+        { "data", 0, DATA_OPERATOR, data_copy,
           data_clone, data_destroy, data_compare, data_read, data_write           },
-        { sizeof(idmef_value_type_id_t), INTEGER_OPERATOR, generic_copy,
-          generic_clone, NULL, generic_compare, enum_read, enum_write, /* enum */ },
-        { 0, 0, NULL, NULL, NULL, NULL, NULL, NULL                                },
-        { 0, OBJECT_OPERATOR, NULL, NULL, NULL, NULL, NULL, NULL                  },
+        { "enum", sizeof(idmef_value_type_id_t), INTEGER_OPERATOR, generic_copy,
+          generic_clone, NULL, generic_compare, enum_read, enum_write,            },
+        { "list", 0, 0, NULL, NULL, NULL, NULL, NULL, NULL                        },
+        { "class", 0, CLASS_OPERATOR, NULL, NULL, NULL, NULL, NULL, NULL          },
 };
 
 
@@ -435,13 +444,19 @@ static const idmef_value_type_operation_t ops_tbl[] = {
 static int is_type_valid(idmef_value_type_id_t type) 
 {
         if ( type < 0 || type >= (sizeof(ops_tbl) / sizeof(*ops_tbl)) )
-                return prelude_error(PRELUDE_ERROR_IDMEF_VALUE_TYPE_UNKNOWN);
+                return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_UNKNOWN, "Unknown IDMEF type id: '%d'", type);
         
         return 0;
 }
 
 
 
+static const char *type_id_to_string(idmef_value_type_id_t type)
+{
+        return ops_tbl[type].name;
+        
+}
+
 
 int idmef_value_type_clone(const idmef_value_type_t *src, idmef_value_type_t *dst)
 {
@@ -519,8 +534,7 @@ int idmef_value_type_read(idmef_value_type_t *dst, const char *buf)
                 return prelude_error(PRELUDE_ERROR_IDMEF_VALUE_TYPE_READ_UNAVAILABLE);
 
         ret = ops_tbl[dst->id].read(dst, buf);
-        
-        return (ret < 0) ? prelude_error(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE) : 0;
+        return (ret < 0) ? ret : 0;
 }
 
 
@@ -570,5 +584,9 @@ int idmef_value_type_check_operator(const idmef_value_type_t *type, idmef_criter
         if ( (~ops_tbl[type->id].operator & op) == 0 )
                 return 0;
         
+        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_CRITERION_UNSUPPORTED_OPERATOR,
+                                     "Object type '%s' does not support operator '%s'",
+                                     type_id_to_string(type->id), idmef_criterion_operator_to_string(op));
+        
         return prelude_error(PRELUDE_ERROR_IDMEF_CRITERION_UNSUPPORTED_OPERATOR);
 }
