@@ -31,7 +31,7 @@
  * This is mostly used by the libprelude detection routine in libprelude.m4
  * to make sure that the header version is consistant with the linked library.
  */
-#define LIBPRELUDE_VERSION "0.9.0"
+#define LIBPRELUDE_VERSION "0.9.7.2"
 
 #include "prelude-inttypes.h"
 #include "common.h"
@@ -46,6 +46,7 @@
 #include "prelude-plugin.h"
 #include "prelude-msg.h"
 #include "prelude-msgbuf.h"
+#include "prelude-timer.h"
 #include "idmef.h"
 
 #ifdef __cplusplus
