@@ -77,17 +77,17 @@ static int get_profile_analyzerid(prelude_client_profile_t *cp)
         
         fd = fopen(filename, "r");
         if ( ! fd )
-                return prelude_error(PRELUDE_ERROR_ANALYZERID_FILE);
+                return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "could not open '%s' for reading", filename);
 
         ptr = fgets(buf, sizeof(buf), fd);
         fclose(fd);
         
         if ( ! ptr )
-                return prelude_error(PRELUDE_ERROR_ANALYZERID_PARSE);
+                return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "could not read analyzerID from '%s'", filename);
         
         ret = sscanf(buf, "%" PRELUDE_PRIu64, &cp->analyzerid);
         if ( ret != 1 )
-                return prelude_error(PRELUDE_ERROR_ANALYZERID_PARSE);
+                return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "'%s' is not a valid analyzerID", buf);
         
         return 0;
 }
@@ -140,7 +140,7 @@ void prelude_client_profile_get_tls_key_filename(prelude_client_profile_t *cp, c
 
 
 /**
- * prelude_client_profile_get_tls_server_filename:
+ * prelude_client_profile_get_tls_server_ca_cert_filename:
  * @cp: pointer on a #prelude_client_profile_t object.
  * @buf: buffer to write the returned filename to.
  * @size: size of @buf.
@@ -161,7 +161,7 @@ void prelude_client_profile_get_tls_server_ca_cert_filename(prelude_client_profi
  * @buf: buffer to write the returned filename to.
  * @size: size of @buf.
  *
- * Writes the filename used to store certificate for @cp private key.
+ * Writes the filename used to store certificate for @cp server.
  * This only apply to @cp receiving connection from analyzer (server).
  */
 void prelude_client_profile_get_tls_server_keycert_filename(prelude_client_profile_t *cp, char *buf, size_t size) 
@@ -171,6 +171,21 @@ void prelude_client_profile_get_tls_server_keycert_filename(prelude_client_profi
 
 
 
+/**
+ * prelude_client_profile_get_tls_server_crl_filename:
+ * @cp: pointer on a #prelude_client_profile_t object.
+ * @buf: buffer to write the returned filename to.
+ * @size: size of @buf.
+ *
+ * Writes the filename used to store CRL for @cp server.
+ * This only apply to @cp receiving connection from analyzer (server).
+ */
+void prelude_client_profile_get_tls_server_crl_filename(prelude_client_profile_t *cp, char *buf, size_t size) 
+{
+        snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/server.crl", cp->name);
+}
+
+
 
 /**
  * prelude_client_profile_get_tls_client_trusted_cert_filename:
