@@ -250,14 +250,19 @@ static void connection_list_destroy(cnx_list_t *clist)
 static void notify_dead(cnx_t *cnx, prelude_error_t error, prelude_bool_t init_time)
 {
         int fd;
+        prelude_error_code_t code;
         cnx_list_t *clist = cnx->parent;
         prelude_connection_pool_t *pool = clist->parent;
 
         if ( cnx->is_dead )
                 return;
+
+        code = prelude_error_get_code(error);
         
-        prelude_log(PRELUDE_LOG_WARN, "%s: connection error with %s: %s. Failover enabled.\n",
-                    prelude_strsource(error), prelude_connection_get_peer_addr(cnx->cnx), prelude_strerror(error));
+        if ( ! init_time || code != PRELUDE_ERROR_PROFILE )
+                prelude_log(PRELUDE_LOG_WARN, "%sconnection error with %s: %s\n",
+                            (pool->flags & PRELUDE_CONNECTION_POOL_FLAGS_FAILOVER) ? "Failover enabled: " : "",
+                            prelude_connection_get_peer_addr(cnx->cnx), prelude_strerror(error));
         
         clist->dead++;
         cnx->is_dead = TRUE;
@@ -295,6 +300,17 @@ static void check_for_data_cb(void *arg)
 }
 
 
+static int failover_save_msg(prelude_failover_t *failover, prelude_msg_t *msg)
+{
+        int ret;
+        
+        ret = prelude_failover_save_msg(failover, msg);
+        if ( ret < 0 )
+                prelude_log(PRELUDE_LOG_WARN, "failover error: %s.\n", prelude_strerror(ret));
+
+        return ret;
+}
+
 
 static void broadcast_message(prelude_msg_t *msg, cnx_t *cnx)
 {
@@ -304,14 +320,14 @@ static void broadcast_message(prelude_msg_t *msg, cnx_t *cnx)
                 return;
         
         if ( prelude_connection_is_alive(cnx->cnx) ) {
-
+                
                 ret = do_send(cnx->cnx, msg);                
                 if ( ret < 0 )
                         notify_dead(cnx, ret, FALSE);
         }
 
         if ( ret < 0 && cnx->failover )
-                prelude_failover_save_msg(cnx->failover, msg);
+                failover_save_msg(cnx->failover, msg);
         
         broadcast_message(msg, cnx->and);
 }
@@ -342,7 +358,7 @@ static int walk_manager_lists(prelude_connection_pool_t *pool, prelude_msg_t *ms
         }
 
         if ( pool->failover )
-                prelude_failover_save_msg(pool->failover, msg);
+                failover_save_msg(pool->failover, msg);
         
         return ret;
 }
@@ -390,7 +406,7 @@ static int failover_flush(prelude_failover_t *failover, cnx_list_t *clist, cnx_t
                         if ( ret < 0 ) {
                                 notify_dead(cnx, ret, FALSE);
                                 if ( cnx->failover )
-                                        prelude_failover_save_msg(cnx->failover, msg);
+                                        failover_save_msg(cnx->failover, msg);
                         }
                 }
 
@@ -424,8 +440,10 @@ static void connection_timer_expire(void *data)
         
         ret = prelude_connection_connect(cnx->cnx, pool->client_profile, pool->permission);
         if ( ret < 0 ) {
-                prelude_perror(ret, "could not connect to %s", prelude_connection_get_peer_addr(cnx->cnx));
-                
+                prelude_log(PRELUDE_LOG_WARN, "%sconnection error with %s: %s\n",
+                            (pool->flags & PRELUDE_CONNECTION_POOL_FLAGS_FAILOVER) ? "Failover enabled: " : "",
+                            prelude_connection_get_peer_addr(cnx->cnx), prelude_strerror(ret));
+                                
                 /*
                  * Connection failed:
                  * expand the current timeout and reset the timer.
@@ -499,7 +517,7 @@ static int new_connection(cnx_t **ncnx, prelude_client_profile_t *cp, cnx_list_t
                 if ( ret < 0 )
                         return ret;
         
-                ret = prelude_failover_new(&new->failover, dirname);        
+                ret = prelude_failover_new(&new->failover, dirname);
                 free(dirname);
                 if ( ret < 0 ) {
                         free(new);
@@ -561,7 +579,7 @@ static int new_connection_from_address(cnx_t **new,
         if ( cret < 0 ) {
                 notify_dead(*new, cret, TRUE);
                 
-                if ( prelude_client_is_setup_needed(cret) )
+                if ( prelude_error_get_code(cret) == PRELUDE_ERROR_PROFILE )
                         return cret;
         }
         
@@ -1150,30 +1168,11 @@ prelude_connection_pool_flags_t prelude_connection_pool_get_flags(prelude_connec
 
 
 
-/**
- * prelude_connection_pool_check_event:
- * @pool: Pointer to a #prelude_connection_pool_t object.
- * @timeout: Time to wait for an event.
- * @event_cb: User provided callback function to call on received events.
- * @extra: Pointer to user specific data provided to @event_cb.
- *
- * This function queries the set of connections available in @pool to see if
- * events are waiting to be handled. If timeout is zero, then this function
- * will return immediatly in case there is no event to be handled.
- *
- * If timeout is -1, this function won't return until an event is available.
- * Otherwise this function will return if there is no event after the specified
- * number of second.
- *
- * For each event, @event_cb is called with the concerned @pool, the provided
- * @extra data, and the @cnx where an event has occured.
- *
- * Returns: The number of handled events.
- */
-int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int timeout,
-                                        int (*event_cb)(prelude_connection_pool_t *pool,
-                                                        prelude_connection_pool_event_t event,
-                                                        prelude_connection_t *cnx, void *extra), void *extra)
+static int connection_pool_check_event(prelude_connection_pool_t *pool, int timeout,
+                                       int (*event_cb)(prelude_connection_pool_t *pool,
+                                                       prelude_connection_pool_event_t event,
+                                                       prelude_connection_t *cnx, void *extra),
+                                       void *extra, prelude_connection_t **conn, prelude_msg_t **outmsg)
 {
         cnx_t *cnx;
         fd_set rfds;
@@ -1197,7 +1196,7 @@ int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int tim
         else if ( ret == 0 )
                 return 0;
         
-        prelude_list_for_each(&pool->int_cnx, tmp) {
+        prelude_list_for_each(&pool->int_cnx, tmp) {                
                 cnx = prelude_list_entry(tmp, cnx_t, list);
                 
                 if ( ! prelude_connection_is_alive(cnx->cnx) )
@@ -1209,7 +1208,28 @@ int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int tim
 
                 global_event |= PRELUDE_CONNECTION_POOL_EVENT_INPUT;
 
-                if ( event_cb ) {
+                i++;
+                if ( conn ) {
+                        *conn = cnx->cnx;
+
+                        if ( outmsg ) {
+                                do {
+                                        ret = prelude_connection_recv(cnx->cnx, outmsg);
+                                } while ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN );
+                        
+                                if ( ret < 0 ) {
+                                        global_event |= PRELUDE_CONNECTION_POOL_EVENT_DEAD;
+                                        notify_dead(cnx, ret, FALSE);
+                                                                
+                                        i--;
+                                        continue;
+                                }
+                        }
+                        
+                        break;
+                }
+                
+                else if ( event_cb ) {
                         ret = event_cb(pool, PRELUDE_CONNECTION_POOL_EVENT_INPUT, cnx->cnx, extra);
                         if ( ret < 0 ) {
                                 global_event |= PRELUDE_CONNECTION_POOL_EVENT_DEAD;
@@ -1217,15 +1237,13 @@ int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int tim
                         }
                 }
 
-                else {
+                else if ( pool->event_handler ) {
                         ret = pool->event_handler(pool, PRELUDE_CONNECTION_POOL_EVENT_INPUT, cnx->cnx);
                         if ( ret < 0 || ! prelude_connection_is_alive(cnx->cnx) ) {
                                 global_event |= PRELUDE_CONNECTION_POOL_EVENT_DEAD;
                                 notify_dead(cnx, ret, FALSE);
                         }
                 }
-                
-                i++;
         }
 
         if ( global_event & pool->wanted_event && pool->global_event_handler )
@@ -1238,6 +1256,66 @@ int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int tim
 }
 
 
+
+/**
+ * prelude_connection_pool_check_event:
+ * @pool: Pointer to a #prelude_connection_pool_t object.
+ * @timeout: Time to wait for an event.
+ * @event_cb: User provided callback function to call on received events.
+ * @extra: Pointer to user specific data provided to @event_cb.
+ *
+ * This function queries the set of connections available in @pool to see if
+ * events are waiting to be handled. If timeout is zero, then this function
+ * will return immediatly in case there is no event to be handled.
+ *
+ * If timeout is -1, this function won't return until an event is available.
+ * Otherwise this function will return if there is no event after the specified
+ * number of second.
+ *
+ * For each event, @event_cb is called with the concerned @pool, the provided
+ * @extra data, and the @cnx where an event has occured.
+ *
+ * Returns: The number of handled events, or a negative value if an error occured.
+ */
+int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int timeout,
+                                        int (*event_cb)(prelude_connection_pool_t *pool,
+                                                        prelude_connection_pool_event_t event,
+                                                        prelude_connection_t *cnx, void *extra), void *extra)
+{
+        return connection_pool_check_event(pool, timeout, event_cb, extra, NULL, NULL);
+}
+
+
+
+/**
+ * prelude_connection_pool_recv:
+ * @pool: Pointer to a #prelude_connection_pool_t object.
+ * @timeout: Time to wait for an event.
+ * @outconn: Pointer where the connection where an event happened should be stored.
+ * @outmsg: Pointer where the next message that will be read should be stored.
+ * 
+ * This function queries the set of connections available in @pool to see if
+ * events are waiting to be handled. If timeout is zero, then this function
+ * will return immediatly in case there is no event to be handled.
+ *
+ * If timeout is -1, this function won't return until an event is available.
+ * Otherwise this function will return if there is no event after the specified
+ * number of second.
+ *
+ * If an event is available, it will be read and store the #prelude_connection_t
+ * object in the @outconn pointer. If @outmsg was specified, the message will be
+ * read and stored in there.
+ *
+ * Returns: The number of handled events (0 or 1) or a negative value if an error occured.
+ */
+int prelude_connection_pool_recv(prelude_connection_pool_t *pool, int timeout, prelude_connection_t **outconn, prelude_msg_t **outmsg)
+{
+        return connection_pool_check_event(pool, timeout, NULL, NULL, outconn, outmsg);
+}
+
+
+
+
 /**
  * prelude_connection_pool_set_data:
  * @pool: Pointer to a #prelude_connection_pool_t object.
