@@ -105,10 +105,13 @@ static int connection_write_msgbuf(prelude_msgbuf_t *msgbuf, prelude_msg_t *msg)
 
 
 
-static void auth_error(prelude_connection_t *cnx, prelude_connection_permission_t reqperms,
-                       prelude_client_profile_t *cp) 
+static int auth_error(prelude_connection_t *cnx,
+                      prelude_connection_permission_t reqperms,
+                      prelude_client_profile_t *cp, prelude_error_t error, const char *fmt, ...) 
 {
-        char *tmp;
+        int ret;
+        va_list ap;
+        char *tmp, buf[1024];
         prelude_string_t *out;
 
         prelude_string_new(&out);
@@ -117,16 +120,22 @@ static void auth_error(prelude_connection_t *cnx, prelude_connection_permission_
         tmp = strrchr(cnx->daddr, ':');
         if ( tmp )
                 *tmp = '\0';
+
+        va_start(ap, fmt);
+        vsnprintf(buf, sizeof(buf), fmt, ap);
+        va_end(ap);
         
-        prelude_log(PRELUDE_LOG_WARN, "\nTLS authentication failed. Please run :\n"
-                    "prelude-adduser register %s \"%s\" %s --uid %d --gid %d\n"
-                    "program on the sensor host to create an account for this sensor.\n\n",
-                    prelude_client_profile_get_name(cp), prelude_string_get_string(out), cnx->daddr,
-                    prelude_client_profile_get_uid(cp), prelude_client_profile_get_gid(cp));
-        
+        ret = prelude_error_verbose_make(prelude_error_get_source(error), prelude_error_get_code(error), "%s.\n\n"
+                 "In order to register this sensor, please run:\n"
+                 "prelude-adduser register %s \"%s\" %s --uid %d --gid %d",
+                 buf, prelude_client_profile_get_name(cp), prelude_string_get_string(out), cnx->daddr,
+                 prelude_client_profile_get_uid(cp), prelude_client_profile_get_gid(cp));
+                 
         prelude_string_destroy(out);
         if ( tmp )
                 *tmp = ':';
+
+        return ret;
 }
 
 
@@ -181,7 +190,7 @@ static int is_tcp_connection_still_established(prelude_io_t *pio)
  */
 static int generic_connect(struct sockaddr *sa, socklen_t salen)
 {
-        int ret, sock;
+        int ret, sock, on = 1;
         
         sock = socket(sa->sa_family, SOCK_STREAM, 0);
 	if ( sock < 0 ) 
@@ -195,6 +204,10 @@ static int generic_connect(struct sockaddr *sa, socklen_t salen)
                 return prelude_error_from_errno(errno);
         }
 
+        ret = setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof(int));
+        if ( ret < 0 )
+                prelude_log(PRELUDE_LOG_ERR, "could not set SO_KEEPALIVE socket option: %s.\n", strerror(errno)); 
+       
         ret = connect(sock, sa, salen);
 	if ( ret < 0 ) {
                 close(sock);
@@ -215,10 +228,8 @@ static int handle_authentication(prelude_connection_t *cnx,
         prelude_string_t *gbuf, *wbuf;
         
         ret = tls_auth_connection(cp, cnx->fd, crypt, &cnx->peer_analyzerid, &cnx->permission);
-        if ( ret < 0 ) {
-                auth_error(cnx, reqperms, cp);
-                return ret;
-        }
+        if ( ret < 0 )
+                return auth_error(cnx, reqperms, cp, ret, "%s", prelude_strerror(ret));
 
         if ( (cnx->permission & reqperms) != reqperms ) {
                 ret = prelude_string_new(&gbuf);
@@ -233,17 +244,16 @@ static int handle_authentication(prelude_connection_t *cnx,
                                 
                 prelude_connection_permission_to_string(cnx->permission, gbuf);
                 prelude_connection_permission_to_string(reqperms, wbuf);
-
-                prelude_log(PRELUDE_LOG_WARN,
-                            "- Insufficient credentials: got \"%s\" but at least \"%s\" required.\n",
-                            prelude_string_get_string(gbuf), prelude_string_get_string(wbuf));
-
+                
+                ret = auth_error(cnx, reqperms, cp, prelude_error(PRELUDE_ERROR_PROFILE),
+                                 "Insufficient credentials: got '%s' but at least '%s' required",
+                                 prelude_string_get_string(gbuf), prelude_string_get_string(wbuf));
+                
                 prelude_string_destroy(gbuf);
                 prelude_string_destroy(wbuf);
-
+                
         err:
-                auth_error(cnx, reqperms, cp);
-                return prelude_error(PRELUDE_ERROR_INSUFFICIENT_CREDENTIALS);
+                return ret;
         }
 
         prelude_log(PRELUDE_LOG_INFO, "- TLS authentication succeed with Prelude Manager.\n");
@@ -259,7 +269,7 @@ static int start_inet_connection(prelude_connection_t *cnx,
                                  prelude_connection_permission_t reqperms, prelude_client_profile_t *profile) 
 {
         socklen_t len;
-        int sock, ret;
+        int sock, ret, tmp;
         struct sockaddr_in addr;
         
         sock = generic_connect(cnx->sa, cnx->salen);
@@ -270,7 +280,10 @@ static int start_inet_connection(prelude_connection_t *cnx,
         
         ret = handle_authentication(cnx, reqperms, profile, 1);
         if ( ret < 0 ) {
-                prelude_io_close(cnx->fd);
+                do {                        
+                        tmp = prelude_io_close(cnx->fd);
+                } while ( tmp < 0 && ! prelude_io_is_error_fatal(cnx->fd, tmp) );
+                
                 return ret;
         }
         
@@ -281,8 +294,8 @@ static int start_inet_connection(prelude_connection_t *cnx,
         len = sizeof(addr);
 
         ret = getsockname(sock, (struct sockaddr *) &addr, &len);
-        if ( ret < 0 ) 
-                prelude_log(PRELUDE_LOG_ERR, "couldn't get connection informations.\n");
+        if ( ret < 0 )
+                ret = prelude_error_verbose(PRELUDE_ERROR_SYSTEM_ERROR, "getsockname failed: %s", strerror(errno));
         else {
                 cnx->saddr = strdup(inet_ntoa(addr.sin_addr));
                 cnx->sport = ntohs(addr.sin_port);
@@ -297,7 +310,7 @@ static int start_inet_connection(prelude_connection_t *cnx,
 static int start_unix_connection(prelude_connection_t *cnx,
                                  prelude_connection_permission_t reqperms, prelude_client_profile_t *profile) 
 {
-        int ret, sock;
+        int ret, sock, tmp;
         
         sock = generic_connect(cnx->sa, cnx->salen);
         if ( sock < 0 )
@@ -307,8 +320,10 @@ static int start_unix_connection(prelude_connection_t *cnx,
         
         ret = handle_authentication(cnx, reqperms, profile, 0);
         if ( ret < 0 )
-                prelude_io_close(cnx->fd);
-                
+                do {
+                        tmp = prelude_io_close(cnx->fd);
+                } while ( tmp < 0 && ! prelude_io_is_error_fatal(cnx->fd, tmp) );
+        
         return ret;
 }
 
@@ -364,8 +379,8 @@ static int close_connection_fd_block(prelude_connection_t *cnx)
         
         do {
                 ret = close_connection_fd(cnx);
-        } while ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN );
-
+        } while ( ret < 0 && ! prelude_io_is_error_fatal(cnx->fd, ret) );
+        
         return ret;
 }
 
@@ -425,9 +440,10 @@ static int do_getaddrinfo(prelude_connection_t *cnx, struct addrinfo **ai, const
         
         ret = getaddrinfo(addr, buf, &hints, ai);
         if ( ret != 0 ) {
-                prelude_log(PRELUDE_LOG_WARN, "could not resolve %s: %s.\n",
-                            addr, (ret == EAI_SYSTEM) ? strerror(errno) : gai_strerror(ret));
-                return prelude_error(PRELUDE_ERROR_CANT_RESOLVE);
+                ret = prelude_error_verbose(PRELUDE_ERROR_CANT_RESOLVE, "could not resolve '%s': %s",
+                                            addr, (ret == EAI_SYSTEM) ? strerror(errno) : gai_strerror(ret));
+                free(addr);
+                return ret;
         }
         
         snprintf(buf, sizeof(buf), "%s:%d", addr, port);
@@ -495,7 +511,7 @@ static int resolve_addr(prelude_connection_t *cnx, const char *addr)
  * connection in a non blocking manner prior prelude_connection_destroy().
  */
 void prelude_connection_destroy(prelude_connection_t *conn) 
-{        
+{
         destroy_connection_fd(conn);
         
         free(conn->daddr);
@@ -528,7 +544,7 @@ int prelude_connection_new(prelude_connection_t **out, const char *addr)
                 if ( ret < 0 ) {
                         prelude_io_destroy(new->fd);
                         free(new);
-                        return prelude_error(PRELUDE_ERROR_CANT_RESOLVE);
+                        return ret;
                 }
         }
         
@@ -597,7 +613,7 @@ int prelude_connection_connect(prelude_connection_t *conn,
 
 
 int prelude_connection_close(prelude_connection_t *cnx) 
-{
+{        
         return close_connection_fd(cnx);
 }
 
@@ -607,7 +623,7 @@ int prelude_connection_close(prelude_connection_t *cnx)
 int prelude_connection_send(prelude_connection_t *cnx, prelude_msg_t *msg) 
 {
         ssize_t ret;
-                
+        
         if ( ! (cnx->state & PRELUDE_CONNECTION_STATE_ESTABLISHED) )
                 return -1;
 
@@ -628,7 +644,7 @@ int prelude_connection_recv(prelude_connection_t *cnx, prelude_msg_t **msg)
 {
         int ret;
         uint8_t tag;
-        
+
         if ( ! (cnx->state & PRELUDE_CONNECTION_STATE_ESTABLISHED) )
                 return -1;
 
@@ -637,15 +653,13 @@ int prelude_connection_recv(prelude_connection_t *cnx, prelude_msg_t **msg)
                 return ret;
 
         tag = prelude_msg_get_tag(*msg);
-        if ( tag == PRELUDE_MSG_IDMEF && !(cnx->permission & PRELUDE_CONNECTION_PERMISSION_IDMEF_READ) ) {
-                prelude_log(PRELUDE_LOG_WARN, "insufficiant credentials for receiving IDMEF message.\n");
-                return prelude_error(PRELUDE_ERROR_INSUFFICIENT_CREDENTIALS);
-        }
+        if ( tag == PRELUDE_MSG_IDMEF && !(cnx->permission & PRELUDE_CONNECTION_PERMISSION_IDMEF_READ) )
+                return prelude_error_verbose(PRELUDE_ERROR_PROFILE,
+                                             "Insufficient credentials for receiving IDMEF message");
         
-        if ( tag == PRELUDE_MSG_OPTION_REQUEST && !(cnx->permission & PRELUDE_CONNECTION_PERMISSION_ADMIN_READ) ) {
-                prelude_log(PRELUDE_LOG_WARN, "insufficiant credentials for receiving IDMEF message.\n");
-                return prelude_error(PRELUDE_ERROR_INSUFFICIENT_CREDENTIALS);
-        }
+        if ( tag == PRELUDE_MSG_OPTION_REQUEST && !(cnx->permission & PRELUDE_CONNECTION_PERMISSION_ADMIN_READ) )
+                return prelude_error_verbose(PRELUDE_ERROR_PROFILE,
+                                             "Insufficient credentials for receiving administrative message");
         
         return ret;
 }
@@ -842,7 +856,7 @@ int prelude_connection_permission_new_from_string(prelude_connection_permission_
                 }
 
                 if ( ! tbl[i].name )
-                        return prelude_error(PRELUDE_ERROR_UNKNOWN_PERMISSION_TYPE);
+                        return prelude_error_verbose(PRELUDE_ERROR_UNKNOWN_PERMISSION_TYPE, "unknown permission type '%s'", tptr);
                 
                 while ( *tmp == ' ' )
                         tmp++;
@@ -858,7 +872,7 @@ int prelude_connection_permission_new_from_string(prelude_connection_permission_
                         else if ( c == ' ' )
                                 break;
                         
-                        else return prelude_error(PRELUDE_ERROR_UNKNOWN_PERMISSION_BIT);
+                        else return prelude_error_verbose(PRELUDE_ERROR_UNKNOWN_PERMISSION_BIT, "unknown permission bit: '%c'", c);
                 }
         }
 
