@@ -57,8 +57,11 @@ static int config_save_value(config_t *cfg, int rtype, prelude_option_t *last, i
                 return -1;
         
         if ( prelude_option_has_optlist(last) ) {
-                
-                snprintf(buf, sizeof(buf), "%s=%s", option, (value) ? value : "default");
+
+                if ( prelude_option_get_type(last) & PRELUDE_OPTION_TYPE_CONTEXT )
+                        snprintf(buf, sizeof(buf), "%s=%s", option, (value) ? value : "default");
+                else
+                        snprintf(buf, sizeof(buf), "%s", option);
                 
                 if ( *prev )
                         free(*prev);
@@ -90,7 +93,7 @@ static int parse_single(void **context, prelude_option_t **last, int is_last_cmd
                         int rtype, const char *option, const char *value, prelude_string_t *out)
 {
         int ret = 0;
-	
+        
         *last = prelude_option_search(*last, option, PRELUDE_OPTION_TYPE_WIDE, 0);
         if ( ! *last ) {
                 prelude_string_sprintf(out, "Unknown option: %s.\n", option);
@@ -102,15 +105,15 @@ static int parse_single(void **context, prelude_option_t **last, int is_last_cmd
         
         else if ( is_last_cmd ) {
 		
-                if ( rtype == PRELUDE_MSG_OPTION_DESTROY ) 
+                if ( rtype == PRELUDE_MSG_OPTION_DESTROY )
                         ret = prelude_option_invoke_destroy(*last, value, out, *context);
                 
                 else if ( rtype == PRELUDE_MSG_OPTION_GET )
                         ret = prelude_option_invoke_get(*last, value, out, *context);
-        
-		else if ( rtype == PRELUDE_MSG_OPTION_COMMIT )
-			ret = prelude_option_invoke_commit(*last, value, out, *context);
-	}
+                
+                else if ( rtype == PRELUDE_MSG_OPTION_COMMIT )
+                        ret = prelude_option_invoke_commit(*last, value, out, *context);
+        }
         
         return ret;
 }
@@ -118,6 +121,40 @@ static int parse_single(void **context, prelude_option_t **last, int is_last_cmd
 
 
 
+static char *option_strsep(char **request)
+{
+        char *start = *request;
+        prelude_bool_t ignore = FALSE;
+
+        if ( ! *request )
+                return NULL;
+        
+        while ( **request ) {
+                if ( ignore == TRUE && **request == ']' )
+                        ignore = FALSE;
+
+                if ( ignore == FALSE && **request == '[' )
+                        ignore = TRUE;
+                
+                if ( ignore == FALSE && **request == '.' ) {
+                        **request = 0;
+                        *request = *request + 1;
+                        return start;
+                }
+                
+                (*request)++;
+        }
+
+        if ( start != *request ) {
+                *request = NULL;
+                return start;
+        }
+
+        return NULL;
+}
+
+
+
 static int parse_request(prelude_client_t *client, int rtype, char *request, prelude_string_t *out)
 {
         config_t *cfg;
@@ -135,7 +172,7 @@ static int parse_request(prelude_client_t *client, int rtype, char *request, pre
         value = request;
         strsep(&value, "=");
         
-        while ( (str = (strsep(&request, "."))) ) {
+        while ( (str = (option_strsep(&request))) ) {
                 
                 if ( ! request ) {
                         last_cmd = 1;
@@ -144,13 +181,13 @@ static int parse_request(prelude_client_t *client, int rtype, char *request, pre
 
                 *pname = 0;
                 *iname = 0;
-                
+
                 ent = ret = sscanf(str, "%255[^[][%255[^]]", pname, iname);
                 if ( ret < 1 ) {
                         prelude_string_sprintf(out, "Error parsing option path");
                         break;
                 }
-
+                
                 ret = parse_single(&context, &last, last_cmd, rtype, pname, (ent == 2) ? iname : ptr, out);
                 if ( ret < 0 )
                         break;
@@ -477,10 +514,10 @@ int prelude_option_recv_reply(prelude_msg_t *msg, uint64_t *source_id,
                         break;
                                                 
                 case PRELUDE_MSG_OPTION_TARGET_ID:
-                        if ( dlen % sizeof(uint64_t) != 0 || dlen < 2 * sizeof(uint64_t) )
+                        if ( dlen % sizeof(uint64_t) != 0 || dlen < (2 * sizeof(uint64_t)) )
                                 return -1;
-                        
-                        *source_id = prelude_extract_uint64(buf + dlen);
+
+                        *source_id = prelude_extract_uint64((unsigned char *) buf + (dlen - sizeof(uint64_t)));
                         break;
 
                 case PRELUDE_MSG_OPTION_LIST:                        
