@@ -134,25 +134,19 @@ static void option_err(int flag, const char *fmt, ...)
 
 
 
-static prelude_option_context_t *search_context(prelude_option_t *opt, const char *name)
+static int option_ret_error(prelude_error_code_t code, prelude_string_t *err, const char *fmt, ...)
 {
         int ret;
-        prelude_list_t *tmp;
-        prelude_option_context_t *ptr;
+        va_list ap;
 
-        if ( ! name || ! *name )
-                name = DEFAULT_INSTANCE_NAME;
-                
-        prelude_list_for_each(&opt->context_list, tmp) {
+        va_start(ap, fmt);
+        
+        prelude_string_vprintf(err, fmt, ap);
+        ret = prelude_error_verbose(code, prelude_string_get_string(err));
 
-                ptr = prelude_list_entry(tmp, prelude_option_context_t, list);
+        va_end(ap);
 
-                ret = strcasecmp(ptr->name, name);
-                if ( ret == 0 )
-                        return ptr;
-        }
-
-        return NULL;
+        return ret;
 }
 
 
@@ -184,6 +178,9 @@ static prelude_option_t *search_option(prelude_option_t *root, const char *optna
         prelude_list_t *tmp;
         prelude_option_t *item, *ret;
 
+        if ( ! root )
+                return NULL;
+
 #if 0
         cmp = cmp_option(root, optname, type);
         if ( cmp == 0 )
@@ -192,7 +189,7 @@ static prelude_option_t *search_option(prelude_option_t *root, const char *optna
         
         prelude_list_for_each(&root->optlist, tmp) {
                 item = prelude_linked_object_get_object(tmp);
-                                
+                
                 if ( walk_children || (! item->longopt && ! item->shortopt) ) {
                         ret = search_option(item, optname, type, walk_children);
                         if ( ret )
@@ -213,9 +210,9 @@ static prelude_bool_t is_an_argument(const char *stuff)
 {
         prelude_option_t *opt;
         size_t len = strlen(stuff);
-        
-        if ( stuff[0] == '-' && (len == 2 || (len > 2 && stuff[1] == '-')) ) {
-                opt = prelude_option_search(NULL, stuff + strspn(stuff, "-"), ~0, TRUE);                
+
+        if ( stuff[0] == '-' && (len == 2 || (len > 2 && stuff[1] == '-')) ) {                
+                opt = prelude_option_search(NULL, stuff + strspn(stuff, "-"), ~0, TRUE);
                 return opt ? FALSE : TRUE;
         }
         
@@ -223,67 +220,25 @@ static prelude_bool_t is_an_argument(const char *stuff)
 }
 
 
-
-static void reorder_argv(int *argc, char **argv, int removed, int *argv_index)
+static int check_option(prelude_option_t *option, const char *arg, prelude_string_t *err)
 {
-        int i;
-        
-        for ( i = removed; (i + 1) < *argc; i++ ) 
-                argv[i] = argv[i + 1];
-
-        (*argc)--;
-        (*argv_index)--;
-}
-
-
-
-
-static int check_option_optarg(const char **outptr, const char *option, const char *arg)
-{
-        if ( arg && is_an_argument(arg) )
-                *outptr = arg;
-        
-        return 0;
-}
-
-
-
-
-static int check_option_reqarg(const char **outptr, const char *option, const char *arg)
-{        
-        if ( ! arg || ! is_an_argument(arg) ) {                
-                fprintf(stderr, "Option %s require an argument.\n", option);
-                return -1;
-        }
-
-        *outptr = arg;
-                
-        return 0;
-}
-
-
-
-static int check_option(prelude_option_t *option, const char **optarg, const char *arg)
-{
-        int ret = -1;
-        
-        *optarg = NULL;
+        int ret = 0;
         
         switch (option->has_arg) {
                 
+        case PRELUDE_OPTION_ARGUMENT_NONE:
+                if ( arg )
+                        ret = option_ret_error(PRELUDE_ERROR_GENERIC, err, "option '%s' does not take an argument", option->longopt);
+                
         case PRELUDE_OPTION_ARGUMENT_OPTIONAL:
-                ret = check_option_optarg(optarg, option->longopt, arg);
                 break;
                 
-        case PRELUDE_OPTION_ARGUMENT_REQUIRED:
-                ret = check_option_reqarg(optarg, option->longopt, arg);
-                break;
-
-        case PRELUDE_OPTION_ARGUMENT_NONE:
-                ret = 0;
+        case PRELUDE_OPTION_ARGUMENT_REQUIRED:                
+                if ( ! arg || ! is_an_argument(arg) )
+                        ret = option_ret_error(PRELUDE_ERROR_GENERIC, err, "option '%s' require an argument", option->longopt);
                 break;
         }
-
+        
         return ret;
 }
 
@@ -299,10 +254,8 @@ static int process_cfg_file(void *context, prelude_list_t *cblist, prelude_optio
         prelude_log_debug(3, "Using configuration file: %s.\n", filename);
         
         ret = config_open(&cfg, filename);
-        if ( ret < 0 ) {
-                prelude_string_sprintf(err, "%s: could not open %s: %s", prelude_strsource(ret), filename, prelude_strerror(ret));
+        if ( ret < 0 )
                 return ret;
-        }
         
         ret = get_missing_options(context, cfg, filename, cblist, optlist, &line, 0, err);
         
@@ -322,15 +275,6 @@ static int do_set(prelude_option_t *opt, const char *value, prelude_string_t *ou
         if ( opt->default_context )
                 *context = opt->default_context;
         
-        if ( opt->type & PRELUDE_OPTION_TYPE_CONTEXT ) {
-                
-                oc = search_context(opt, value);
-                if ( oc ) {
-                        *context = oc->data;
-                        return 0;
-                }
-        }
-                
         if ( ! opt->set )
                 return 0;
 
@@ -339,16 +283,19 @@ static int do_set(prelude_option_t *opt, const char *value, prelude_string_t *ou
         
         ret = opt->set(opt, value, out, *context);        
         if ( ret < 0 ) {
-                if ( prelude_string_is_empty(out) )
-                        prelude_string_sprintf(out, "could not set option %s: %s",
-                                               opt->longopt, prelude_strerror(ret));
+                if ( prelude_string_is_empty(out) ) {
+                        prelude_string_sprintf(out, "error while setting option '%s'", opt->longopt);
+
+                        if ( prelude_error_get_code(ret) != PRELUDE_ERROR_GENERIC )
+                                prelude_string_sprintf(out, ": %s", prelude_strerror(ret));
+                }
                 
                 return ret;
         }
         
         if ( opt->type & PRELUDE_OPTION_TYPE_CONTEXT ) {
                                 
-                oc = search_context(opt, value);
+                oc = prelude_option_search_context(opt, value);
                 if ( ! oc )
 		        return -1;
                 
@@ -471,7 +418,6 @@ static int get_missing_options(void *context, config_t *cfg, const char *filenam
                                prelude_option_t *rootlist, unsigned int *line, int depth, prelude_string_t *err)
 {
         int ret;
-        const char *argptr;
         prelude_option_t *opt;
         struct cb_list *cbitem;
         char *section = NULL, *entry = NULL, *value = NULL;
@@ -516,12 +462,14 @@ static int get_missing_options(void *context, config_t *cfg, const char *filenam
                 
                 if ( section && ! entry ) {
 
-                        if ( cblist ) {
-                                ret = check_option(opt, &argptr, value);
-                                if ( ret < 0 )
-                                        return ret;
+                        if ( cblist ) {                                
+                                ret = check_option(opt, value, err);
+                                if ( ret < 0 ) {
+                                        const char *tmp = _prelude_thread_get_error();
+                                        return prelude_error_verbose(prelude_error_get_code(ret), "%s:%d: %s", filename, *line, tmp);
+                                }
                                 
-                                ret = call_option_cb(context, &cbitem, cblist, opt, argptr, err, SET_FROM_CFG);
+                                ret = call_option_cb(context, &cbitem, cblist, opt, value, err, SET_FROM_CFG);
                                 if ( ret < 0 ) 
                                         return ret;
                         }
@@ -529,16 +477,18 @@ static int get_missing_options(void *context, config_t *cfg, const char *filenam
                         ret = get_missing_options(context, cfg, filename, (cblist) ?
                                                   &cbitem->children : NULL, opt, line, depth + 1, err);
                         if ( ret < 0 )
-                                return -1;
+                                return ret;
                         
                 }
 
                 else if ( cblist ) {
-                        ret = check_option(opt, &argptr, value);                        
-                        if ( ret < 0 )
-                                return ret;
+                        ret = check_option(opt, value, err);
+                        if ( ret < 0 ) {
+                                const char *tmp = _prelude_thread_get_error();
+                                return prelude_error_verbose(prelude_error_get_code(ret), "%s:%d: %s", filename, *line, tmp);
+                        }
                         
-                        ret = call_option_cb(context, &cbitem, cblist, opt, argptr, err, SET_FROM_CFG);
+                        ret = call_option_cb(context, &cbitem, cblist, opt, value, err, SET_FROM_CFG);
                         if ( ret < 0 ) 
                                 return ret;
                 }
@@ -548,63 +498,143 @@ static int get_missing_options(void *context, config_t *cfg, const char *filenam
 }
 
 
+static void remove_argv(int argc, char **argv, char **unhandled, int *unhandled_index, int removed)
+{
+        int i;
 
-static int parse_argument(void *context, prelude_list_t *cb_list, prelude_option_t *optlist,
-                          int *argc, char **argv, int *argv_index, int depth, prelude_string_t *err)
+        unhandled[(*unhandled_index)++] = argv[removed];
+        
+        for ( i = removed; (i + 1) < argc; i++ ) 
+                argv[i] = argv[i + 1];
+}
+
+
+
+static int parse_option(prelude_option_t *root_optlist, prelude_option_t *optlist,
+                        int *argc, char **argv, int *argv_index,
+                        char **unhandled, int *unhandled_index, char **option, const char **value,
+                        prelude_option_t **opt, prelude_bool_t *ignore)
+{
+        size_t len;
+        int ret, optch = 0;
+        prelude_option_t *tmp;
+        
+        *value = NULL;
+        
+        *option = argv[(*argv_index)++];
+        len = strlen(*option);
+        
+        if ( **option != '-' || len == 1 ) {
+                remove_argv(*argc, argv, unhandled, unhandled_index, --(*argv_index));
+                return 0;
+        }
+        
+        if ( strcmp(*option, "--") == 0 ) {                
+                for ( ret = *argv_index; ret < *argc; ret++ )
+                        remove_argv(*argc, argv, unhandled, unhandled_index, ret);
+
+                return -1; /* break */
+        }
+                
+        while ( **option == '-' ) {
+                (*option)++;
+                optch++;
+        }
+        
+        if ( ! isalnum((int) **option) )
+                return 0;
+        
+        /*
+         * FIXME: handle consecutive shortopt
+         */
+        
+        *value = strchr(*option, '=');
+        if ( ! *value )
+                *option = strdup(*option);
+        else {
+                *option = strndup(*option, strcspn(*option, "="));
+                (*value)++;
+        }
+
+        *opt = search_option(optlist, *option, PRELUDE_OPTION_TYPE_CLI, 0);                
+        if ( root_optlist != _prelude_generic_optlist &&
+             (tmp = search_option(_prelude_generic_optlist, *option, ~0, FALSE)) ) {
+                *opt = tmp;
+                *ignore = TRUE;
+        }
+        
+        if ( ! *opt )
+                return 1;
+
+        if ( (*opt)->has_arg == PRELUDE_OPTION_ARGUMENT_REQUIRED && ! *value && (*argv_index < *argc)  )
+                *value = argv[(*argv_index)++];
+        
+        return 1;
+}
+
+
+
+static int parse_argument(void *context, prelude_list_t *cb_list,
+                          prelude_option_t *root_optlist, prelude_option_t *optlist,
+                          int *argc, char **argv, int *argv_index,
+                          char **unhandled, int *unhandled_index,
+                          int depth, prelude_string_t *err, prelude_bool_t ignore)
 {
         int ret;
+        char *option;
+        const char *arg;
         prelude_option_t *opt;
         struct cb_list *cbitem;
-        const char *arg, *old, *argptr;
         
-        while ( *argv_index < *argc ) {
-                                
-                old = arg = argv[(*argv_index)++];
+        while ( *argv_index < (*argc - *unhandled_index) ) {                
+                ret = parse_option(root_optlist, optlist, argc, argv, argv_index,
+                                   unhandled, unhandled_index, &option, &arg, &opt, &ignore);
+                if ( ret < 0 )
+                        break;
                 
-                if ( *arg != '-' )
+                else if ( ret == 0 )
                         continue;
-                                
-                while ( *arg == '-' ) arg++;
 
-                if ( ! isalnum((int) *arg) )
-                        continue;
-                     
-                opt = search_option(optlist, arg, PRELUDE_OPTION_TYPE_CLI, 0);                
-                if ( ! opt ) {                        
+                if ( ! opt ) {
                         if ( depth ) {
+                                free(option);
                                 (*argv_index)--;
                                 return 0;
                         }
                         
-                        option_err(PRELUDE_OPTION_WARNING_OPTION, "invalid option -- \"%s\" (%d).\n", arg, depth);
+                        remove_argv(*argc, argv, unhandled, unhandled_index, --(*argv_index));
+                        option_err(PRELUDE_OPTION_WARNING_OPTION, "Invalid option -- \"%s\" (%d).\n", option, depth);
+                        free(option);
+                        
                         continue;
                 }
-                
-                reorder_argv(argc, argv, *argv_index - 1, argv_index);
-                
-                ret = check_option(opt, &argptr, (*argv_index < *argc) ? argv[*argv_index] : NULL);
-                if ( ret < 0 ) 
-                        return -1;
 
-                if ( argptr )
-                        reorder_argv(argc, argv, *argv_index, argv_index);
+                free(option);
                 
-                ret = call_option_cb(context, &cbitem, cb_list, opt, argptr, err, SET_FROM_CLI);
+                ret = check_option(opt, arg, err);
                 if ( ret < 0 )
                         return ret;
                 
+                if ( ! ignore ) {
+                        ret = call_option_cb(context, &cbitem, cb_list, opt, arg, err, SET_FROM_CLI);
+                        if ( ret < 0 )
+                                return ret;
+                }
+                
                 /*
                  * If the option we just found have sub-option.
                  * Try to match the rest of our argument against them.
                  */
                 if ( ! prelude_list_is_empty(&opt->optlist) ) {
                         
-                        ret = parse_argument(context, &cbitem->children, opt,
-                                             argc, argv, argv_index, depth + 1, err);
+                        ret = parse_argument(context, &cbitem->children, root_optlist, opt,
+                                             argc, argv, argv_index, unhandled, unhandled_index, depth + 1, err, ignore);
 
                         if ( ret < 0 )
                                 return ret;
                 }
+                
+                ignore = FALSE;
         }
         
         return 0;
@@ -616,15 +646,29 @@ static int parse_argument(void *context, prelude_list_t *cb_list, prelude_option
 static int get_option_from_optlist(void *context, prelude_option_t *optlist,
                                    const char **filename, int *argc, char **argv, prelude_string_t **err)
 {
+        char **unhandled;
         prelude_list_t cblist;
-        int argv_index = 1, ret = 0;
+        int i, unhandled_index = 0, argv_index = 1, ret = 0;
                   
         prelude_list_init(&cblist);
         
-        if ( argc ) {                
-                ret = parse_argument(context, &cblist, optlist, argc, argv, &argv_index, 0, *err);
+        if ( argc ) {
+                unhandled = malloc(*argc * sizeof(*unhandled));
+                if ( ! unhandled )
+                        return prelude_error_from_errno(errno);
+                
+                ret = parse_argument(context, &cblist, optlist, optlist, argc, argv, &argv_index,
+                                     unhandled, &unhandled_index, 0, *err, FALSE);
+                
+                for ( i = 0; i < unhandled_index; i++)
+                        argv[*argc - unhandled_index + i] = unhandled[i];
+
+                free(unhandled);
+                
                 if ( ret < 0 )
                         return ret;
+
+                unhandled_index += ret;
         }
         
         if ( filename && *filename ) {                     
@@ -637,7 +681,7 @@ static int get_option_from_optlist(void *context, prelude_option_t *optlist,
         if ( ret < 0 )
                 return ret;
         
-        return ret;
+        return *argc - unhandled_index;
 }
 
 
@@ -663,8 +707,8 @@ static int get_option_from_optlist(void *context, prelude_option_t *optlist,
  *
  * if @option is NULL, all system option will be matched against argc, and argv.
  *
- * Returns: 0 if parsing the option succeed (including the case where one of
- * the callback returned -1 to request interruption of parsing), -1 if an error occured.
+ * Returns: The index of the first unhandled parameter if option parsing succeeded,
+ * or a negative value if an error occured.
  */
 int prelude_option_read(prelude_option_t *option, const char **filename,
                         int *argc, char **argv, prelude_string_t **err, void *context) 
@@ -690,10 +734,7 @@ int prelude_option_read(prelude_option_t *option, const char **filename,
                 *err = NULL;
         }
         
-        if ( ret < 0 )
-                return ret;
-
-        return 0;
+        return ret;
 }
 
 
@@ -908,25 +949,40 @@ static int get_max_char(const char *line, int descoff)
 
 
 
+static int print_space(FILE *fd, size_t num)
+{
+        char buf[3];
+        size_t len, totlen = 0;
+        
+        do {
+                len = MIN(sizeof(buf), num - totlen);
+                
+                memset(buf, ' ', len);
+                fwrite(buf, 1, len, fd);
+                totlen += len;
+                
+        } while ( totlen < num );
+       
+        return num;
+}
+
+
 static void print_wrapped(FILE *fd, const char *line, int descoff) 
 {
-        int max, i = 0, j;
+        int max;
+        size_t size = 0, i;
         
-        while ( 1 ) {
-                max = get_max_char(&line[i], descoff);
-                
-                while ( max-- >= 0 ) {
-                        
-                        if ( line[i] == '\0' ) {
-                                fputc('\n', fd);
-                                return;
-                        } else
-                                fputc(line[i++], fd);
+        while ( 1 ) {             
+                max = get_max_char(line + size, descoff);
+                               
+                size += i= fwrite(line + size, 1, max, fd);                
+                if ( line[size] == '\0' ) {
+                        fputc('\n', fd);
+                        return;
                 }
-                        
+
                 fputc('\n', fd);
-                for ( j = 0; j < descoff; j++ )
-                        fputc(' ', fd);
+                print_space(fd, descoff - 1);
         }
 }
 
@@ -947,25 +1003,44 @@ static void print_options(FILE *fd, prelude_option_t *root, prelude_option_type_
                  */
                 if ( opt->type == PRELUDE_OPTION_TYPE_ROOT )
                         print_options(fd, opt, type, descoff, depth);
-                else {                          
+                else {
+                        i = 0;
                         if ( type && ! (opt->type & type) ) 
                                 continue;
 
-                        for ( i = 0; i < depth; i++ )
-                                fprintf(fd, "  ");
+                        if ( depth ) {
+                                i += depth;
+                                print_space(fd, depth * 2);
+                        }
                         
                         if ( ! prelude_list_is_empty(&opt->optlist) )
                                 fputc('\n', fd);
                         
                         if ( opt->shortopt != 0 )
-                                i += fprintf(fd, "-%c ", opt->shortopt);
-                        
-                        if ( opt->longopt )
-                                i += fprintf(fd, "--%s ", opt->longopt);
-                        
-                        while ( i++ < descoff )
-                                fputc(' ', fd);
+                                i += fprintf(fd, "-%c%s", opt->shortopt, opt->longopt ? ", " : " ");
+
+                        if ( opt->longopt ) {
+                                if ( ! opt->shortopt && prelude_list_is_empty(&opt->optlist) )
+                                        i += fprintf(fd, "    ");
+                                
+                                i += fprintf(fd, "--%s", opt->longopt);
+                                if ( (opt->has_arg == PRELUDE_OPTION_ARGUMENT_OPTIONAL ||
+                                      opt->has_arg == PRELUDE_OPTION_ARGUMENT_REQUIRED) &&
+                                     ! prelude_list_is_empty(&opt->optlist) )        
+                                        i += fprintf(fd, "=[INAME] ");
+                                
+                                else if ( opt->has_arg == PRELUDE_OPTION_ARGUMENT_REQUIRED )
+                                        i += fprintf(fd, "=ARG ");
+                                
+                                else if ( opt->has_arg == PRELUDE_OPTION_ARGUMENT_OPTIONAL )
+                                        i += fprintf(fd, "=[ARG] ");
+                                else
+                                        i+= fprintf(fd, " ");
+                        }
                         
+                        if ( i < descoff )
+                                i += print_space(fd, descoff - i);
+                                                
                         if ( opt->description )
                                 print_wrapped(fd, opt->description, depth + descoff);
                         else
@@ -992,7 +1067,8 @@ static void print_options(FILE *fd, prelude_option_t *root, prelude_option_type_
  * If @opt is NULL, then the root of the option list is used.
  */
 void prelude_option_print(prelude_option_t *opt, prelude_option_type_t type, int descoff, FILE *fd) 
-{        
+{
+        descoff += 9;
         print_options(fd, opt ? opt : root_optlist, type, descoff, 0);
 }
 
@@ -1035,26 +1111,26 @@ void prelude_option_destroy(prelude_option_t *option)
         prelude_list_t *tmp, *bkp;
         prelude_option_t *opt;
         
-        if ( ! option )
+        if ( ! option ) {
                 option = root_optlist;
+                root_optlist = NULL;
+        }
         
         prelude_list_for_each_safe(&option->optlist, tmp, bkp) {
                 opt = prelude_linked_object_get_object(tmp);
                 prelude_option_destroy(opt);
         }
+        
+        if ( option->value )
+                free(option->value);
 
-        if ( option ) {                
-                if ( option->value )
-                        free(option->value);
-
-                prelude_list_for_each_safe(&option->context_list, tmp, bkp)
-                        prelude_option_context_destroy(prelude_list_entry(tmp, prelude_option_context_t, list));
+        prelude_list_for_each_safe(&option->context_list, tmp, bkp)
+                prelude_option_context_destroy(prelude_list_entry(tmp, prelude_option_context_t, list));
                 
-                if ( ! prelude_list_is_empty(&option->_list) )
-                        prelude_linked_object_del((prelude_linked_object_t *) option);
+        if ( ! prelude_list_is_empty(&option->_list) )
+                prelude_linked_object_del((prelude_linked_object_t *) option);
                 
-                free(option);
-        }
+        free(option);
 }
 
 
@@ -1225,17 +1301,13 @@ void *prelude_option_get_data(prelude_option_t *opt)
 
 
 int prelude_option_invoke_set(prelude_option_t *opt, const char *value, prelude_string_t *err, void **context)
-{        
-        if ( opt->has_arg == PRELUDE_OPTION_ARGUMENT_NONE && value ) {
-                prelude_string_sprintf(err, "%s does not take an argument", opt->longopt);
-                return -1;
-        }
+{
+        int ret;
+        
+        ret = check_option(opt, value, err);        
+        if ( ret < 0 )
+                return ret;
         
-        if ( opt->has_arg == PRELUDE_OPTION_ARGUMENT_REQUIRED && ! value ) {
-                prelude_string_sprintf(err, "%s require an argument", opt->longopt);
-                return -1;
-        }
-
         prelude_log_debug(3, "opt=%s value=%s\n", opt->longopt, value ? value : "");
         
         return do_set(opt, value, err, context);
@@ -1255,18 +1327,17 @@ int prelude_option_invoke_commit(prelude_option_t *opt, const char *ctname, prel
                 context = opt->default_context;
         
 	if ( opt->type & PRELUDE_OPTION_TYPE_CONTEXT ) {
-		oc = search_context(opt, ctname);
-		if ( ! oc ) {
-			prelude_string_sprintf(value, "could not find option with context %s[%s]",
-                                               opt->longopt, ctname);
-			return -1;
-		}
+		oc = prelude_option_search_context(opt, ctname);
+		if ( ! oc )
+                        return option_ret_error(PRELUDE_ERROR_GENERIC, value,
+                                                "could not find option with context %s[%s]", opt->longopt, ctname);
 		context = oc->data;
 	}
 	
         ret = opt->commit(opt, value, context);
 	if ( ret < 0 && prelude_string_is_empty(value) )
-		prelude_string_sprintf(value, "commit failed for %s", opt->longopt);
+                ret = option_ret_error(prelude_error_get_code(ret), value,
+                                       "could not find option with context %s[%s]", opt->longopt, ctname);
 	
 	return ret;
 }
@@ -1278,30 +1349,24 @@ int prelude_option_invoke_destroy(prelude_option_t *opt, const char *ctname, pre
 	int ret;
 	prelude_option_context_t *oc = NULL;
 	
-        if ( ! opt->destroy ) {
-                prelude_string_sprintf(value, "%s does not support destruction", opt->longopt);
-		return -1;
-	}
+        if ( ! opt->destroy )
+                return option_ret_error(PRELUDE_ERROR_GENERIC, value, "%s does not support destruction", opt->longopt);
 
         if ( opt->default_context )
                 context = opt->default_context;
         
 	if ( opt->type & PRELUDE_OPTION_TYPE_CONTEXT ) {
-		oc = search_context(opt, ctname);
-		if ( ! oc ) {
-			prelude_string_sprintf(value, "could not find option with context %s[%s]",
-                                               opt->longopt, ctname);
-			return -1;
-		}
+		oc = prelude_option_search_context(opt, ctname);
+		if ( ! oc )
+                        return option_ret_error(PRELUDE_ERROR_GENERIC, value,
+                                                "could not find option with context %s[%s]", opt->longopt, ctname);
                 
 		context = oc->data;
 	}
         
         ret = opt->destroy(opt, value, context);
-	if ( ret < 0 && prelude_string_is_empty(value) ) {
-		prelude_string_sprintf(value, "destruction for %s[%s] failed", opt->longopt, ctname);
-		return -1;
-	}
+	if ( ret < 0 && prelude_string_is_empty(value) )
+                return option_ret_error(PRELUDE_ERROR_GENERIC, value, "destruction for %s[%s] failed", opt->longopt, ctname);
 	
 	if ( oc ) 
 		prelude_option_context_destroy(oc);
@@ -1324,7 +1389,7 @@ int prelude_option_invoke_get(prelude_option_t *opt, const char *ctname, prelude
                 context = opt->default_context;
         
 	if ( opt->type & PRELUDE_OPTION_TYPE_CONTEXT ) {
-		oc = search_context(opt, ctname);
+		oc = prelude_option_search_context(opt, ctname);
 		if ( ! oc ) {
 			prelude_string_sprintf(value, "could not find option with context %s[%s]",
                                                opt->longopt, ctname);
@@ -1556,3 +1621,26 @@ prelude_option_t *prelude_option_search(prelude_option_t *parent, const char *na
 {
         return search_option(parent ? parent : root_optlist, name, type, walk_children);
 }
+
+
+
+prelude_option_context_t *prelude_option_search_context(prelude_option_t *opt, const char *name)
+{
+        int ret;
+        prelude_list_t *tmp;
+        prelude_option_context_t *ptr;
+
+        if ( ! name || ! *name )
+                name = DEFAULT_INSTANCE_NAME;
+                
+        prelude_list_for_each(&opt->context_list, tmp) {
+
+                ptr = prelude_list_entry(tmp, prelude_option_context_t, list);
+
+                ret = strcasecmp(ptr->name, name);
+                if ( ret == 0 )
+                        return ptr;
+        }
+
+        return NULL;
+}
