@@ -24,6 +24,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <pthread.h>
@@ -48,9 +49,9 @@
 
 #else
 
-#  if !PTHREAD_IN_USE_DETECTION_HARD
-#   define __prelude_thread_in_use() 1
-#  endif
+# if !PTHREAD_IN_USE_DETECTION_HARD
+#  define __prelude_thread_in_use() TRUE
+# endif
 
 # endif
 
@@ -70,14 +71,32 @@
 
 
 #define THR_FUNC_NORETURN(func)                         \
-        if ( use_thread ) return func; else return
+        if ( use_thread ) func
 
 
 
-static prelude_bool_t use_thread = FALSE;
+static pthread_key_t thread_error_key;
+static char *shared_error_buffer = NULL;
+static prelude_bool_t use_thread = FALSE, need_init = TRUE;
 
 
 
+static void thread_error_key_destroy(void *value)
+{
+        free(value);
+}
+
+
+static void thread_init_if_needed(void)
+{
+        if ( ! need_init )
+                return;
+        
+        pthread_key_create(&thread_error_key, thread_error_key_destroy);
+        need_init = FALSE;
+}
+
+
 int prelude_thread_atfork(void (*prepare)(void), void (*parent)(void), void (*child)(void))
 {
         THR_FUNC(pthread_atfork(prepare, parent, child));
@@ -186,18 +205,36 @@ int prelude_thread_sigmask(int how, const sigset_t *newmask, sigset_t *oldmask)
 }
 
 
-
 /*
  *
  */
 int prelude_thread_init(void *nil)
 {
+        thread_init_if_needed();
         use_thread = TRUE;
         return 0;
 }
 
 
 
+/*
+ *
+ */
+void _prelude_thread_deinit(void)
+{
+        if ( use_thread ) {
+                pthread_key_delete(thread_error_key);
+                need_init = TRUE;
+        }
+        
+        else if ( shared_error_buffer ) {
+                free(shared_error_buffer);
+                shared_error_buffer = NULL;
+        }
+}
+
+
+
 /*
  * Pthread stubs detection code for Solaris/Hp from GnuLib.
  */
@@ -239,14 +276,50 @@ prelude_bool_t _prelude_thread_in_use(void)
 {
         static prelude_bool_t tested = FALSE;
         
-        if ( tested )
+        if ( tested ) {
+                thread_init_if_needed();
                 return use_thread;
+        }
         
         use_thread = __prelude_thread_in_use();
         tested = TRUE;
-        
+
         prelude_log(PRELUDE_LOG_DEBUG, "[init] thread used=%d\n", use_thread);
+        thread_init_if_needed();
         
         return use_thread;
 }
 
+
+int _prelude_thread_set_error(const char *error)
+{
+        char *previous;
+
+        if ( ! use_thread ) {                
+                if ( shared_error_buffer )
+                        free(shared_error_buffer);
+                
+                shared_error_buffer = strdup(error);
+        }
+
+        else {
+                previous = pthread_getspecific(thread_error_key);
+                if ( previous )
+                        free(previous);
+                
+                pthread_setspecific(thread_error_key, strdup(error));
+        }
+        
+        return 0;
+}
+
+
+
+const char *_prelude_thread_get_error(void)
+{        
+        if ( use_thread )
+                return pthread_getspecific(thread_error_key);
+        else
+                return shared_error_buffer;
+}
+
