@@ -89,37 +89,55 @@ static int read_auth_result(prelude_io_t *fd)
 
 static int verify_certificate(gnutls_session session)
 {
-	int ret;
+        time_t now;
+	int ret, status, alert = 0;
+        const prelude_error_code_t code = PRELUDE_ERROR_PROFILE;
         
-	ret = gnutls_certificate_verify_peers(session);
+	ret = gnutls_certificate_verify_peers2(session, &status);
 	if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_WARN, "- TLS certificate error: %s.\n", gnutls_strerror(ret));
-                return -1;
+                gnutls_alert_send_appropriate(session, ret);
+                return prelude_error_verbose(code, "TLS certificate verification failed: %s", gnutls_strerror(ret));
+        }        
+        
+        if ( status & GNUTLS_CERT_INVALID ) {
+                alert = GNUTLS_A_BAD_CERTIFICATE;
+                ret = prelude_error_verbose(code, "TLS server certificate is NOT trusted");
         }
         
-	if ( ret == GNUTLS_E_NO_CERTIFICATE_FOUND ) {
-		prelude_log(PRELUDE_LOG_WARN, "- TLS certificate error: server did not send any certificate.\n");
-		return -1;
-	}
-
-        if ( ret & GNUTLS_CERT_SIGNER_NOT_FOUND) {
-		prelude_log(PRELUDE_LOG_WARN, "- TLS certificate error: server certificate issuer is unknown.\n");
-                return -1;
+        else if ( status & GNUTLS_CERT_REVOKED ) {
+                alert = GNUTLS_A_CERTIFICATE_REVOKED;
+                ret = prelude_error_verbose(code, "TLS server certificate was revoked");
         }
         
-        if ( ret & GNUTLS_CERT_INVALID ) {
-                prelude_log(PRELUDE_LOG_WARN, "- TLS certificate error: server certificate is NOT trusted.\n");
-                return -1;
+        else if ( status & GNUTLS_CERT_SIGNER_NOT_FOUND) {
+                alert = GNUTLS_A_UNKNOWN_CA;
+                ret = prelude_error_verbose(code, "TLS server certificate issuer is unknown");
+        }
+        
+        else if ( status & GNUTLS_CERT_SIGNER_NOT_CA ) {
+                alert = GNUTLS_A_CERTIFICATE_UNKNOWN;
+                ret = prelude_error_verbose(code, "TLS server certificate issuer is not a CA");
         }
 
-        if ( ret & GNUTLS_CERT_SIGNER_NOT_CA ) {
-                prelude_log(PRELUDE_LOG_WARN, "- TLS certificate error: server certificate issuer is not a CA.\n");
-                return -1;
+#ifdef GNUTLS_CERT_INSECURE_ALGORITHM
+        else if ( status & GNUTLS_CERT_INSECURE_ALGORITHM ) {
+                alert = GNUTLS_A_INSUFFICIENT_SECURITY;
+                ret = prelude_error_verbose(code, "TLS server certificate use insecure algorithm");
         }
+#endif
 
-        prelude_log(PRELUDE_LOG_INFO, "- TLS certificate: server certificate is trusted.\n");
+        now = time(NULL);
+        
+        if ( gnutls_certificate_activation_time_peers(session) > now )
+                ret = prelude_error_verbose(code, "TLS server certificate not yet activated.\n");
 
-        return 0;
+        if ( gnutls_certificate_expiration_time_peers(session) < now )
+                ret = prelude_error_verbose(code, "TLS server certificate expired.\n");
+        
+        if ( ret < 0 )
+                gnutls_alert_send(session, GNUTLS_AL_FATAL, alert);
+        
+        return ret;
 }
 
 
@@ -141,7 +159,11 @@ static int handle_gnutls_error(gnutls_session session, int ret)
                 return -1;
         }
 
-        return (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED) ? 0 : ret;
+        if ( ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED )
+                return 0;
+
+        gnutls_alert_send_appropriate(session, ret);
+        return ret;
 }
 
 
@@ -181,17 +203,16 @@ int tls_auth_connection(prelude_client_profile_t *cp, prelude_io_t *io, int cryp
         do {
                 ret = gnutls_handshake(session);
         } while ( ret < 0 && handle_gnutls_error(session, ret) == 0 );
-        
+
         if ( ret < 0 ) {
                 gnutls_deinit(session);
-                prelude_log(PRELUDE_LOG_WARN, "- TLS handshake failed: %s.\n", gnutls_strerror(ret));
-                return prelude_error(PRELUDE_ERROR_TLS_HANDSHAKE);
+                return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "TLS handshake failed: %s", gnutls_strerror(ret));
         }
 
         ret = verify_certificate(session);        
         if ( ret < 0 ) {
                 gnutls_deinit(session);
-                return prelude_error(PRELUDE_ERROR_TLS_INVALID_CERTIFICATE);
+                return ret;
         }
         
         prelude_io_set_tls_io(io, session);
@@ -211,17 +232,17 @@ int tls_auth_connection(prelude_client_profile_t *cp, prelude_io_t *io, int cryp
         if ( ! crypt ) {
                 
                 do {
-                        ret = gnutls_bye(session, GNUTLS_SHUT_RDWR);
-                } while ( ret < 0 && (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN) );
+                        ret = gnutls_bye(session, GNUTLS_SHUT_RDWR);                        
+                } while ( ret < 0 && handle_gnutls_error(session, ret) == 0 );
                 
                 if ( ret < 0 )
-                        prelude_log(PRELUDE_LOG_WARN, "gnutls bye failed with error: %s.\n", gnutls_strerror(ret));
+                        ret = prelude_error_verbose(PRELUDE_ERROR_TLS, "TLS bye failed: %s", gnutls_strerror(ret));
                 
                 gnutls_deinit(session);
                 prelude_io_set_sys_io(io, fd);
         }
         
-        return 0;
+        return ret;
 }
 
 
@@ -235,10 +256,9 @@ int tls_auth_init(prelude_client_profile_t *cp, gnutls_certificate_credentials *
                 gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
         
         ret = gnutls_global_init();
-        if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_ERR, "GnuTLS initialization failed: %s.\n", gnutls_strerror(ret));
-                return prelude_error_make(PRELUDE_ERROR_SOURCE_CLIENT, PRELUDE_ERROR_TLS);
-        }
+        if ( ret < 0 )
+                return prelude_error_verbose_make(PRELUDE_ERROR_SOURCE_CLIENT, PRELUDE_ERROR_TLS,
+                                                  "TLS initialization failed: %s", gnutls_strerror(ret));
         
         prelude_client_profile_get_tls_key_filename(cp, keyfile, sizeof(keyfile));
         prelude_client_profile_get_tls_client_keycert_filename(cp, certfile, sizeof(certfile));
@@ -246,10 +266,9 @@ int tls_auth_init(prelude_client_profile_t *cp, gnutls_certificate_credentials *
         gnutls_certificate_allocate_credentials(cred);
         
         ret = access(certfile, F_OK);
-        if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_ERR, "access to %s failed: %s.\n", certfile, strerror(errno));
-                return prelude_error_make(PRELUDE_ERROR_SOURCE_CLIENT, PRELUDE_ERROR_TLS_CERTIFICATE_FILE);
-        }
+        if ( ret < 0 )
+                return prelude_error_verbose_make(PRELUDE_ERROR_SOURCE_CLIENT, PRELUDE_ERROR_PROFILE,
+                                                  "access to %s failed: %s", certfile, strerror(errno));
         
         ret = tls_certificates_load(keyfile, certfile, *cred);
         if ( ret < 0 )
@@ -258,10 +277,9 @@ int tls_auth_init(prelude_client_profile_t *cp, gnutls_certificate_credentials *
         prelude_client_profile_get_tls_client_trusted_cert_filename(cp, certfile, sizeof(certfile));
         
         ret = gnutls_certificate_set_x509_trust_file(*cred, certfile, GNUTLS_X509_FMT_PEM);
-        if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_ERR, "could not set x509 trust file '%s': %s.\n", certfile, gnutls_strerror(ret));
-                return prelude_error_make(PRELUDE_ERROR_SOURCE_CLIENT, PRELUDE_ERROR_TLS_CERTIFICATE_PARSE);
-        }
+        if ( ret < 0 )
+                return prelude_error_verbose_make(PRELUDE_ERROR_SOURCE_CLIENT, PRELUDE_ERROR_PROFILE,
+                                                  "could not set x509 trust file '%s': %s", certfile, gnutls_strerror(ret));
         
         return 0;
 }
