@@ -1,9 +1,9 @@
 /*****
 *
-* Copyright (C) 2001 Yoann Vandoorselaere <yoann@prelude-ids.org>
-* All Rights Reserved
+* Copyright (C) 2001,2002,2003,2004,2005 PreludeIDS Technologies. All Rights Reserved.
+* Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
-* This file is part of the Prelude program.
+* This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
@@ -19,40 +19,40 @@
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
-* Written by Yoann Vandoorselaere <yoann@prelude-ids.org>
-*
 *****/
 
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "list.h"
-#include "prelude-log.h"
+#include "libmissing.h"
+#include "prelude-list.h"
+#include "prelude-error.h"
 #include "variable.h"
 
 
 typedef struct {
-        struct list_head list;
+        prelude_list_t list;
         char *variable;
         char *value;
 } variable_t;
 
 
 
-static LIST_HEAD(variable_list);
+static PRELUDE_LIST(variable_list);
 
 
 
 static variable_t *search_entry(const char *variable) 
 {
         int ret;
-        struct list_head *tmp;
+        prelude_list_t *tmp;
         variable_t *item = NULL;
         
-        list_for_each(tmp, &variable_list) {
-                item = list_entry(tmp, variable_t, list);
+        prelude_list_for_each(&variable_list, tmp) {
+                item = prelude_list_entry(tmp, variable_t, list);
 
                 ret = strcasecmp(item->variable, variable);
 
@@ -71,15 +71,13 @@ static int create_entry(char *variable, char *value)
         variable_t *item;
 
         item = malloc(sizeof(*item));
-        if ( ! item ) {
-                log(LOG_ERR, "failed to allocate memory.\n");
-                return -1;
-        }
+        if ( ! item )
+                return prelude_error_from_errno(errno);
 
         item->variable = variable;
         item->value = value;
 
-        list_add(&item->list, &variable_list);
+        prelude_list_add(&item->list, &variable_list);
 
         return 0;
 }
@@ -150,7 +148,7 @@ int variable_unset(const char *variable)
         if ( ! item )
                 return -1;
 
-        list_del(&item->list);
+        prelude_list_del(&item->list);
         
         free(item->variable);
         free(item->value);
